/***************************************************************
Copyright (C) DOS Lab, Dept. of CS & E, IIT Madras.		*
								*
This program is free software; you can redistribute it and/or	*
modify it under the terms of the GNU General Public License	*
as published by the Free Software Foundation; either version 2	*
of the License, or (at your option) any later version.		*
								*
This program is distributed in the hope that it will be useful,	*
but WITHOUT ANY WARRANTY; without even the implied warranty of	*
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the	*
GNU General Public License for more details.			*
								*
You should have received a copy of the GNU General Public License*	
along with this program; if not, write to Prof. D. Janakiram, 	*
DOS Lab, Department of CS & E, Indian Institute of Technology Madras,* 
Chennai - 600036, India. Alternatively, you can send a mail to 	*
d.janakiram@cs.iitm.ernet.in.					*	
*****************************************************************/
/*
 * Main program for the steady state pipeline
 * Refer juSteadyState.h for details
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <sys/time.h>
#include <math.h>
#include "juSteadyState.h"

struct timeval currentTime, startTime;
int isourcepDPD, idestpDPD; // the source and destination dist. pipe descriptors
int isourcenDPD, idestnDPD; // the source and dest pipe desc for the next node
int inumNodes;
int idone; // flag indicating steady state
char ssourcepPipe[60], sdestpPipe[60]; // Dist. Pipes we read from and write to the previous node respectively
char sdestnPipe[60], ssourcenPipe[60]; // dist pipes for the next node
char slocalNodeName[30];

long    sec, usec;

double   flocalMatrix[INUMGRIDS][2]; // the state of the rod as seen by the local node
int ibeginRange, iendRange; // the we compute the locations flocalMatrix[i]
// where beginRange <= i <= iendRange
double   fcolumnToSend[INUMGRIDS];

/*
 * doComputation is the main steady state equilibrium solver
 * It assumes that the rod's previous state has been read by the local node from the previous
 * node in the pipeline. This previous state is put in the first column of flocalMatrix
 */
void doComputation() {
    int i, j;
    int prev,next;
    double fpreviousValue;
 
    idone = 1;
    
    for (i = ibeginRange; i <= iendRange; i++) {
	    prev = i - 1;
    	
        if( prev < 0 )
    	    prev = INUMGRIDS - 1 ;
	    
        next = i + 1;
    	
        if( next >= INUMGRIDS )
	        next = 0;
        
	    // remember that the first and the last grid
	    // are in the heat bath, and their vals. don't change
        fpreviousValue = flocalMatrix[i][0];
           
	    if( (!i)||(i == INUMGRIDS-1))
	        flocalMatrix[i][1] = flocalMatrix[i][0];
	    else 
	        flocalMatrix [i][1] =  steadyState(flocalMatrix[i][0], flocalMatrix[prev][0], flocalMatrix[next][0]);

        if (fpreviousValue != flocalMatrix[i][1])
             idone = 0;
    }
   
    // now copy the last timestep into the first one.. for the next iteration
    for( i = ibeginRange; i <= iendRange; i++ )
      flocalMatrix[i][0] = flocalMatrix[i][1];

    // echo a portion of the state
    /*fprintf(stderr, "Rod state : \n");
    for (i = 0; i < 20; i++)
      fprintf(stderr, " %f |", flocalMatrix[i][0]);
    fprintf(stderr, "%s", "...");
    for (i = INUMGRIDS-20; i < INUMGRIDS; i++)
      fprintf(stderr, " %f |", flocalMatrix[i][0]);*/
 

} // doComputation

/*
 * initializePipes creates and opens the distributed pipes through which we read/write
 * the steady state data
 */
void initializePipes(void) {
    int  ipipePos, i;
    

    // find out the range within which to do the computation
    // each node calculates between the ranges
    // ipipepos * (NUMGRIDS/NUMNODES) to (ipipepos+1) * NUMGRIDS/NUMNODES - 1;
    ibeginRange = 0;
    iendRange = INUMGRIDS - 1;

    fprintf(stderr, "Computing for %d grids\n", INUMGRIDS);
    fprintf( stderr, "my range is %d to %d\n", ibeginRange, iendRange );

    gettimeofday( &currentTime, NULL );
    memcpy (&startTime, &currentTime, sizeof(struct timeval));
    
    fprintf( stderr, "[%3d.%6d] Begin computations\n", currentTime.tv_sec, currentTime.tv_usec );
    fprintf( stderr, "CONFIRMATION : [%3d.%6d] Begin computations\n", startTime.tv_sec, startTime.tv_usec );
    
    flocalMatrix[0][0] = FT2;              // Initialize the rod's state at the first node:
    flocalMatrix[INUMGRIDS - 1][0] = FT1;  // All the internal grids of the rod are
    for (i = 1; i < INUMGRIDS - 2; i++)            // set to 0.0 while the ends are in the
      flocalMatrix[i][0] = 0.0;           // heat baths (FT1, FT2).
    
    // start doing the computation
    doComputation();
      
} // initializePipes

/*
 * steadyState is the central steadyState computation : have a dummy for now - just takes the average
 *
double   steadyState (double farg1, double farg2, double farg3) {
  double i;

  for( i = 0.0; i < 1.0; i += 0.05 )
    i += 0.005;
  return fabs(fabs(farg2) + (fabs(farg3)  + fabs(farg1) - fabs(farg2) * 2.0)/1.93576);
}
 */
double steadyState (double farg1, double farg2, double farg3) {
  //  return (farg1+farg2+farg3)/3;
  double i;
  for( i = 0.0; i < 1.0; i += 0.004 ){
    i += 0.0005;
    i /= 1.0;
  }
  return fabs(1.0 * fabs(farg2) + 1.0 * (fabs(farg3) + 1.0 * fabs(farg1) - 1.0 * fabs(farg2) * 2.0)/3.1756);
}

int main() {
    int i,inumIterations, first;
    char *finddot;

    idone = 0;
    
    first = 1;

    initializePipes(); // initialize all necessary pipes on the local machine

    for (inumIterations = 0; inumIterations<999 ; inumIterations++) { // one doComputation already done in initializePipes
        // read the rod state sent by the previous node and the next node in the pipeline through ssourcePipe's
        if( !(inumIterations % 50 )){
    	    gettimeofday( &currentTime, NULL );
        	fprintf( stderr, "[%3d.%6d] Begin iteration %d\n", currentTime.tv_sec, currentTime.tv_usec, inumIterations );
        }

      
        // start doing the computation
        doComputation();
    }

    gettimeofday( &currentTime, NULL );
    sec = currentTime.tv_sec - startTime.tv_sec;
    usec = currentTime.tv_usec - startTime.tv_usec;
    if (usec < 0) {
        sec--;
        usec += 1000000;
    }

    fprintf( stderr, "[%3d.%6d] Computation complete in %d iterations\n", currentTime.tv_sec, currentTime.tv_usec, inumIterations+1 );    // see what the results are
    fprintf(stderr, "\n\nEXECUTION TIME : \n\n%d seconds, %d useconds\n\n", sec, usec);
    
    fprintf(stderr, "The state of the rod between points %d and %d: |", ibeginRange, iendRange);
    for (i = ibeginRange; i <= iendRange; i++)
      fprintf(stderr, " %f |", flocalMatrix[i][1]);
    
    return 0;
} // main
