/* Copyright (C) DOS Lab, Dept. of CS & E, IIT Madras.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to Prof. D. Janakiram, 
DOS Lab, Department of CS & E, Indian Institute of Technology Madras, 
Chennai - 600036, India. Alternatively, you can send a mail to 
d.janakiram@cs.iitm.ernet.in. */

#include <rpc/types.h>
#include <rpc/xdr.h>

#include "GCNetLevel.h"
#include "GCError.h"
#include "GCHost.h"
#include "GCDefines.h"
#include "GCMesg.h"
#include "GCXDRRoutines.h"

char *g_pszProgramName;


/*********************************************************************************************************************
 * Write Fucntions / Encoders
 *********************************************************************************************************************/

/************************************************************************************************************
 * Description: 
 * 	Sends an XDR Format Message.
 * 	
 * Arguments:
 * 	sock - Socket Descriptor on which message has to be sent
 *     	a_pstMessage - Message to be Sent
 *     	
 * Return:
 *   	Error Code / 1
 *   	
 ************************************************************************************************************/

int WriteXDRMessage(sock, a_pstMessage)
	int sock; 
     	PMESSAGE a_pstMessage;
{
  
	int l_nErrorCode;

	/* Send the Message Type */
	
	if((l_nErrorCode = WriteOnHost(sock, (char *)&a_pstMessage->m_nMessageType, sizeof(a_pstMessage->m_nMessageType))) < 0)
	{
      	    return l_nErrorCode;
	}
  
	/* Send the Message Length */
	if((l_nErrorCode = WriteOnHost(sock, (char *)&a_pstMessage->m_nXDRDataLength, sizeof(a_pstMessage->m_nXDRDataLength))) < 0)
	{
      	    return l_nErrorCode;
	}
  
	/* Send the Message */
	if((l_nErrorCode = WriteOnHost(sock, (char *)a_pstMessage->m_pXDRData, ntohs(a_pstMessage->m_nXDRDataLength))) < 0)
	{
      	    return l_nErrorCode;
    	}

	return 1;
}


/************************************************************************************************************
 * Description : 
 *              sends a "signed char" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which message has to be sent
 *              a_nMessageID - Message Identifier to be written
 *              a_pDataElement - pointer to the data element to be sent
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host, TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int WriteXDRSignedChar(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

	MESSAGE l_stWriteMessage;

	int l_nErrorCode;

	GC_CALLOC(l_stWriteMessage.m_pXDRData, char, a_nSize);

	if(a_nWhichHost == TCP_HOST)
	{
  		/* Create the memory for the XDR format data representation */
		xdrmem_create(&l_stWriteMessage.m_XDRVariable, l_stWriteMessage.m_pXDRData, a_nSize, XDR_ENCODE);
      
      		/* Convert to the XDR Data representation */
		if(xdr_char(&l_stWriteMessage.m_XDRVariable, (signed char *)a_pDataElement) == FALSE)
		{
			GCLogError("XDR Encoding Signed character error\n");
			GC_FREE(l_stWriteMessage.m_pXDRData);
			return XDR_ERROR;
		}
	}
	
	else if(a_nWhichHost == UNIXSTREAM_HOST)
	{
		GC_COPY(a_pDataElement, l_stWriteMessage.m_pXDRData, a_nSize);
      	}

	/* Set the length of the message */
	l_stWriteMessage.m_nXDRDataLength = htons(a_nSize);
					 
	/* Set the Message Id */
	l_stWriteMessage.m_nMessageType = htons(a_nMessageID);

	/* Send the Data to the other end */
	l_nErrorCode = WriteXDRMessage(sock, &l_stWriteMessage);
 
	/* Delete the Memory allocated for storing the XDR data format */
	GC_FREE(l_stWriteMessage.m_pXDRData);

	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              sends an "unsigned char" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which message has to be sent
 *              a_nMessageID - Message Identifier to be written
 *              a_pDataElement - pointer to the data element to be sent
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int WriteXDRUnSignedChar(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
     	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

	MESSAGE l_stWriteMessage;

	int l_nErrorCode;

	GC_CALLOC(l_stWriteMessage.m_pXDRData, char, a_nSize);
	
	if(a_nWhichHost == TCP_HOST)
	{
    		/* Create the memory for the XDR format data representation */
	      	xdrmem_create(&l_stWriteMessage.m_XDRVariable, l_stWriteMessage.m_pXDRData, a_nSize, XDR_ENCODE);
      
      	 	/* Convert to the XDR Data representation */
      		if(xdr_u_char(&l_stWriteMessage.m_XDRVariable, (unsigned char *)a_pDataElement) == FALSE)
		{
			GCLogError("XDR Encoding unsigned character Error\n");
			GC_FREE(l_stWriteMessage.m_pXDRData);
			return XDR_ERROR;
		}
	}
	
	else if(a_nWhichHost == UNIXSTREAM_HOST)
	{
		GC_COPY(a_pDataElement, l_stWriteMessage.m_pXDRData, a_nSize);
	}

	/* Set the length of the message */
	l_stWriteMessage.m_nXDRDataLength = htons(a_nSize);
					 
	/* Set the Message Id */
	l_stWriteMessage.m_nMessageType = htons(a_nMessageID);

	/* Send the Data to the other end */
	l_nErrorCode = WriteXDRMessage(sock, &l_stWriteMessage);

	/* Delete the Memory allocated for storing the XDR data format */
	GC_FREE(l_stWriteMessage.m_pXDRData);

	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              sends a "signed short integer" value in XDR format
 *              
 * Arguments   :
 *  		sock - Socket descriptor on which message has to be sent
 *              a_nMessageID - Message Identifier to be written
 *              a_pDataElement - pointer to the data element to be sent
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int WriteXDRSignedShortInteger(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
     	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

	MESSAGE l_stWriteMessage;

	int l_nErrorCode;

	GC_CALLOC(l_stWriteMessage.m_pXDRData, char, a_nSize);
	
	if(a_nWhichHost == TCP_HOST)
	{
  
		/* Create the memory for the XDR format data representation */
		xdrmem_create(&l_stWriteMessage.m_XDRVariable, l_stWriteMessage.m_pXDRData, a_nSize, XDR_ENCODE);
      
		/* Convert to the XDR Data representation */
		if(xdr_short(&l_stWriteMessage.m_XDRVariable, (signed short *)a_pDataElement) == FALSE)
		{
			GCLogError("XDR Encoding signed short integer Error\n");
			GC_FREE(l_stWriteMessage.m_pXDRData);
			return XDR_ERROR;
		}
	}
	
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(a_pDataElement, l_stWriteMessage.m_pXDRData, a_nSize);
      	}
	
	/* Set the length of the message */
	l_stWriteMessage.m_nXDRDataLength = htons(a_nSize);
					 
	/* Set the Message Id */
	l_stWriteMessage.m_nMessageType = htons(a_nMessageID);

	/* Send the Data to the other end */
	l_nErrorCode = WriteXDRMessage(sock, &l_stWriteMessage);

	/* Delete the Memory allocated for storing the XDR data format */
	GC_FREE(l_stWriteMessage.m_pXDRData);

	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              sends an "unsigned short integer" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which message has to be sent
 *              a_nMessageID - Message Identifier to be written
 *              a_pDataElement - pointer to the data element to be sent
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int WriteXDRUnSignedShortInteger(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

	MESSAGE l_stWriteMessage;

	int l_nErrorCode;

	GC_CALLOC(l_stWriteMessage.m_pXDRData, char, a_nSize);
	
	if(a_nWhichHost == TCP_HOST)
	{
  		/* Create the memory for the XDR format data representation */
		xdrmem_create(&l_stWriteMessage.m_XDRVariable, l_stWriteMessage.m_pXDRData, a_nSize, XDR_ENCODE);
      
		/* Convert to the XDR Data representation */
		if(xdr_u_short(&l_stWriteMessage.m_XDRVariable, (unsigned short int *)a_pDataElement) == FALSE)
		{
			GCLogError("XDR Encoding unsigned short integer Error\n");
			GC_FREE(l_stWriteMessage.m_pXDRData);
			return XDR_ERROR;
		}
	}
	
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(a_pDataElement, l_stWriteMessage.m_pXDRData, a_nSize);
      	}
	
	/* Set the length of the message */
	l_stWriteMessage.m_nXDRDataLength = htons(a_nSize);
					 
	/* Set the Message Id */
	l_stWriteMessage.m_nMessageType = htons(a_nMessageID);

	/* Send the Data to the other end */
	l_nErrorCode = WriteXDRMessage(sock, &l_stWriteMessage);

	/* Delete the Memory allocated for storing the XDR data format */
	GC_FREE(l_stWriteMessage.m_pXDRData);

	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              sends a "signed integer" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which message has to be sent
 *              a_nMessageID - Message Identifier to be written
 *              a_pDataElement - pointer to the data element to be sent
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int WriteXDRSignedInteger(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
     	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{
   	
	MESSAGE l_stWriteMessage;

	int l_nErrorCode;

	GC_CALLOC(l_stWriteMessage.m_pXDRData, char, a_nSize);
	
	if(a_nWhichHost == TCP_HOST)
	{
  	  	/* Create the memory for the XDR format data representation */
	  	xdrmem_create(&l_stWriteMessage.m_XDRVariable, l_stWriteMessage.m_pXDRData, a_nSize, XDR_ENCODE);
      
 	  	/* Convert to the XDR Data representation */
	  	if(xdr_int(&l_stWriteMessage.m_XDRVariable, (signed int *)a_pDataElement) == FALSE)
	   	{
			GCLogError("XDR Encoding signed integer Error\n");
			GC_FREE(l_stWriteMessage.m_pXDRData);
			return XDR_ERROR;
	   	} 
	}
	
	else if(a_nWhichHost == UNIXSTREAM_HOST)
		GC_COPY(a_pDataElement, l_stWriteMessage.m_pXDRData, a_nSize);

	/* Set the length of the message */
	l_stWriteMessage.m_nXDRDataLength = htons(a_nSize);
					 
	/* Set the Message Id */
	l_stWriteMessage.m_nMessageType = htons(a_nMessageID);

	/* Send the Data to the other end */
	l_nErrorCode = WriteXDRMessage(sock, &l_stWriteMessage);

	/* Delete the Memory allocated for storing the XDR data format */
	GC_FREE(l_stWriteMessage.m_pXDRData);

	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              sends an "unsigned integer" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which message has to be sent
 *              a_nMessageID - Message Identifier to be written
 *              a_pDataElement - pointer to the data element to be sent
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int WriteXDRUnSignedInteger(sock, a_nMessageID,  a_pDataElement, a_nSize, a_nWhichHost)
     	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

	MESSAGE l_stWriteMessage;

	int l_nErrorCode;

	GC_CALLOC(l_stWriteMessage.m_pXDRData, char, a_nSize);

	if(a_nWhichHost == TCP_HOST)
	{
  
      		/* Create the memory for the XDR format data representation */
		xdrmem_create(&l_stWriteMessage.m_XDRVariable, l_stWriteMessage.m_pXDRData, a_nSize, XDR_ENCODE);
      
		/* Convert to the XDR Data representation */
		if(xdr_u_int(&l_stWriteMessage.m_XDRVariable, (unsigned int *)a_pDataElement) == FALSE)
		{
			GCLogError("XDR Encoding unsigned integer Error\n");
			GC_FREE(l_stWriteMessage.m_pXDRData);
			return XDR_ERROR;
		}
	}
	
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(a_pDataElement, l_stWriteMessage.m_pXDRData, a_nSize);
      	}

	/* Set the length of the message */
	l_stWriteMessage.m_nXDRDataLength = htons(a_nSize);
					 
	/* Set the Message Id */
	l_stWriteMessage.m_nMessageType = htons(a_nMessageID);

	/* Send the Data to the other end */
	l_nErrorCode = WriteXDRMessage(sock, &l_stWriteMessage);

	/* Delete the Memory allocated for storing the XDR data format */
	GC_FREE(l_stWriteMessage.m_pXDRData);

	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              sends a "signed long integer" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which message has to be sent
 *              a_nMessageID - Message Identifier to be written
 *              a_pDataElement - pointer to the data element to be sent
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int WriteXDRSignedLongInteger(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
     	int sock;
      	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

	MESSAGE l_stWriteMessage;

	int l_nErrorCode;

	GC_CALLOC(l_stWriteMessage.m_pXDRData, char, a_nSize);

	if(a_nWhichHost == TCP_HOST)
	{
       	 	/* Create the memory for the XDR format data representation */
		xdrmem_create(&l_stWriteMessage.m_XDRVariable, l_stWriteMessage.m_pXDRData, a_nSize, XDR_ENCODE);
      
		/* Convert to the XDR Data representation */
		if(xdr_long(&l_stWriteMessage.m_XDRVariable, (signed long int *)a_pDataElement) == FALSE)
		{
			GCLogError("XDR Encoding signed long integer Error\n");
			GC_FREE(l_stWriteMessage.m_pXDRData);
			return XDR_ERROR;
		}
	}
	
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(a_pDataElement, l_stWriteMessage.m_pXDRData, a_nSize);
      	}
	
	/* Set the length of the message */
	l_stWriteMessage.m_nXDRDataLength = htons(a_nSize);
					 
	/* Set the Message Id */
	l_stWriteMessage.m_nMessageType = htons(a_nMessageID);

	/* Send the Data to the other end */
	l_nErrorCode = WriteXDRMessage(sock, &l_stWriteMessage);

	/* Delete the Memory allocated for storing the XDR data format */
	GC_FREE(l_stWriteMessage.m_pXDRData);

	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              sends an "unsigned long integer" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which message has to be sent
 *              a_nMessageID - Message Identifier to be written
 *              a_pDataElement - pointer to the data element to be sent
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int WriteXDRUnSignedLongInteger(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

	MESSAGE l_stWriteMessage;

	int l_nErrorCode;

	GC_CALLOC(l_stWriteMessage.m_pXDRData, char, a_nSize);

	if(a_nWhichHost == TCP_HOST)
	{
  		/* Create the memory for the XDR format data representation */
		xdrmem_create(&l_stWriteMessage.m_XDRVariable, l_stWriteMessage.m_pXDRData, a_nSize, XDR_ENCODE);
      
		/* Convert to the XDR Data representation */
		if(xdr_u_long(&l_stWriteMessage.m_XDRVariable, (unsigned long int *)a_pDataElement) == FALSE)
		{
			GCLogError("XDR Encoding unsigned long integer Error\n");
			GC_FREE(l_stWriteMessage.m_pXDRData);
			return XDR_ERROR;
		}
	}
	
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(a_pDataElement, l_stWriteMessage.m_pXDRData, a_nSize);
      	}

	/* Set the length of the message */
	l_stWriteMessage.m_nXDRDataLength = htons(a_nSize);
					 
	/* Set the Message Id */
	l_stWriteMessage.m_nMessageType = htons(a_nMessageID);

	/* Send the Data to the other end */
	l_nErrorCode = WriteXDRMessage(sock, &l_stWriteMessage);

	/* Delete the Memory allocated for storing the XDR data format */
	GC_FREE(l_stWriteMessage.m_pXDRData);

	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              sends a "string" in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which message has to be sent
 *              a_nMessageID - Message Identifier to be written
 *              a_pDataElement - pointer to the data element to be sent
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int WriteXDRString(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

	MESSAGE l_stWriteMessage;
  
	char *l_pszPointerToTheString;

	int l_nErrorCode;
	unsigned int l_nBytesToBeEncoded;

	GC_CALLOC(l_stWriteMessage.m_pXDRData, char, LENGTH_FIELD_FOR_XDR_STRING + a_nSize);
  
	if(a_nWhichHost == TCP_HOST)
	{
      	  	/* Create the memory for the XDR format data representation */
	  	xdrmem_create(&l_stWriteMessage.m_XDRVariable, l_stWriteMessage.m_pXDRData, LENGTH_FIELD_FOR_XDR_STRING + a_nSize, XDR_ENCODE);
      
	  	l_pszPointerToTheString = (char *)a_pDataElement;
      	  	l_nBytesToBeEncoded = a_nSize;
	  
	  /* Convert to the XDR Data representation */
	  /* convert each of them to XDR bytes and then send the data */
	  	if(xdr_string(&l_stWriteMessage.m_XDRVariable, (char **)&l_pszPointerToTheString, a_nSize + LENGTH_FIELD_FOR_XDR_STRING) == FALSE)
		{
			GCLogError("XDR Encoding String Error\n");
			GC_FREE(l_stWriteMessage.m_pXDRData);
			return XDR_ERROR;
   	    	}
	}
	
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(a_pDataElement, l_stWriteMessage.m_pXDRData, a_nSize);
      	}

	/* Set the length of the message */
	l_stWriteMessage.m_nXDRDataLength = htons(a_nSize +  LENGTH_FIELD_FOR_XDR_STRING);
					 
	/* Set the Message Id */
	l_stWriteMessage.m_nMessageType = htons(a_nMessageID);

	/* Send the Data to the other end */
	l_nErrorCode = WriteXDRMessage(sock, &l_stWriteMessage);

	/* Delete the Memory allocated for storing the XDR data format */
	GC_FREE(l_stWriteMessage.m_pXDRData);

	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              sends a "float" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which message has to be sent
 *              a_nMessageID - Message Identifier to be written
 *              a_pDataElement - pointer to the data element to be sent
 *              a_nSize - size of each data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int WriteXDRFloat(sock, a_nMessageID,  a_pDataElement, a_nSize, a_nWhichHost)
     	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

	MESSAGE l_stWriteMessage;

	int l_nErrorCode;

	GC_CALLOC(l_stWriteMessage.m_pXDRData, char, a_nSize);

	if(a_nWhichHost == TCP_HOST)
	{
  
      		/* Create the memory for the XDR format data representation */
		xdrmem_create(&l_stWriteMessage.m_XDRVariable, l_stWriteMessage.m_pXDRData, a_nSize, XDR_ENCODE);
      
		/* Convert to the XDR Data representation */
		if(xdr_float(&l_stWriteMessage.m_XDRVariable, (float *)a_pDataElement) == FALSE)
		{
			GCLogError("XDR Encoding float Error\n");
			GC_FREE(l_stWriteMessage.m_pXDRData);
			return XDR_ERROR;
		}
	}
	
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(a_pDataElement, l_stWriteMessage.m_pXDRData, a_nSize);
      	}

	/* Set the length of the message */
	l_stWriteMessage.m_nXDRDataLength = htons(a_nSize);
					 
	/* Set the Message Id */
	l_stWriteMessage.m_nMessageType = htons(a_nMessageID);

	/* Send the Data to the other end */
	l_nErrorCode = WriteXDRMessage(sock, &l_stWriteMessage);

	/* Delete the Memory allocated for storing the XDR data format */
	GC_FREE(l_stWriteMessage.m_pXDRData);

	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              sends a "double" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which message has to be sent
 *              a_nMessageID - Message Identifier to be written
 *              a_pDataElement - pointer to the data element to be sent
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int WriteXDRDouble(sock, a_nMessageID,  a_pDataElement, a_nSize, a_nWhichHost)
     	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

	MESSAGE l_stWriteMessage;

	int l_nErrorCode;

	GC_CALLOC(l_stWriteMessage.m_pXDRData, char, a_nSize);

	if(a_nWhichHost == TCP_HOST)
	{
  	 	/* Create the memory for the XDR format data representation */
		xdrmem_create(&l_stWriteMessage.m_XDRVariable, l_stWriteMessage.m_pXDRData, a_nSize, XDR_ENCODE);
      
		/* Convert to the XDR Data representation */
		if(xdr_double(&l_stWriteMessage.m_XDRVariable, (double *)a_pDataElement) == FALSE)
		{
			GCLogError("XDR Encoding double Error\n");
			GC_FREE(l_stWriteMessage.m_pXDRData);
			return XDR_ERROR;
		}
	}
	
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(a_pDataElement, l_stWriteMessage.m_pXDRData, a_nSize);
      	}

	/* Set the length of the message */
	l_stWriteMessage.m_nXDRDataLength = htons(a_nSize);
					 
	/* Set the Message Id */
	l_stWriteMessage.m_nMessageType = htons(a_nMessageID);

	/* Send the Data to the other end */
	l_nErrorCode = WriteXDRMessage(sock, &l_stWriteMessage);

	/* Delete the Memory allocated for storing the XDR data format */
	GC_FREE(l_stWriteMessage.m_pXDRData);

	return l_nErrorCode;
}



/*********************************************************************************************************************
 * Read Functions / Decodders
 *********************************************************************************************************************/

/************************************************************************************************************
 * Description : 
 *              Reads an XDR Format Message
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which the read has to be done
 *              a__pstMessage - Message to be read
 *
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int ReadXDRMessage(sock, a_pstMessage)
  	int sock;
  	PMESSAGE a_pstMessage;
{

	int l_nErrorCode;
  
	/* Read the Message Type */
	if((l_nErrorCode = ReadOnHost(sock, (char *)&a_pstMessage->m_nMessageType, sizeof(a_pstMessage->m_nMessageType))) < 0)
	{
		printf(" mesg type ..error\n");
		return l_nErrorCode;
	}
  
	/* Read the Message Length */
	if((l_nErrorCode = ReadOnHost(sock, (char *)&a_pstMessage->m_nXDRDataLength, sizeof(a_pstMessage->m_nXDRDataLength))) < 0)
	{
		printf(" mesg length error..\n");
		return l_nErrorCode;
	}
  
	/* Read the Message */
	if((l_nErrorCode = ReadOnHost(sock, (char *)a_pstMessage->m_pXDRData, ntohs(a_pstMessage->m_nXDRDataLength))) < 0)
	{
		printf(" mesg data..error..\n");
		return l_nErrorCode;
	}

	return 1;
}


/************************************************************************************************************
 * Description : 
 *              Read a "signed char" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which the read has to be done
 *              a_nMessageID - Message Identifier to be read
 *              a_pDataElement - pointer to the data element to be read
 *              a_nSize - size of each data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *
 ************************************************************************************************************/

int ReadXDRSignedChar(sock, a_nMessageID,  a_pDataElement, a_nSize, a_nWhichHost)
	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

	MESSAGE l_stReadMessage;

	int l_nErrorCode;

	GC_CALLOC(l_stReadMessage.m_pXDRData, char, a_nSize);

	if((l_nErrorCode = ReadXDRMessage(sock, &l_stReadMessage)) < 0)
	{
		/* Delete the Memory allocated for storing the XDR data format */
		GC_FREE(l_stReadMessage.m_pXDRData);
		return l_nErrorCode;
	}

	if(a_nWhichHost == TCP_HOST)
	{
  		/* Create the memory for the XDR format data representation */
		xdrmem_create(&l_stReadMessage.m_XDRVariable, l_stReadMessage.m_pXDRData, a_nSize, XDR_DECODE);
      
		/* Convert to the XDR Data representation */
		if(xdr_char(&l_stReadMessage.m_XDRVariable, (signed char *)a_pDataElement) == FALSE)
		{
			GCLogError("XDR Decoding signed character Error\n");
			GC_FREE(l_stReadMessage.m_pXDRData);
			return XDR_ERROR;
		}
    	}
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(l_stReadMessage.m_pXDRData, a_pDataElement, a_nSize);
      	}

	/* Delete the Memory allocated for storing the XDR data format */
	GC_FREE(l_stReadMessage.m_pXDRData);

	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              Reads an "unsigned char" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which the read has to be done
 *              a_nMessageID - Message Identifier to be read
 *              a_pDataElement - pointer to the data element to be read
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int ReadXDRUnSignedChar(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
     	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

	MESSAGE l_stReadMessage;

	int l_nErrorCode;

	GC_CALLOC(l_stReadMessage.m_pXDRData, char, a_nSize);
  
	/* Read the XDR Message */
  	if((l_nErrorCode = ReadXDRMessage(sock, &l_stReadMessage)) < 0)
    	{
      	 	/* Delete the Memory allocated for storing the XDR data format */
		GC_FREE(l_stReadMessage.m_pXDRData);
		return l_nErrorCode;
    	}

	if(a_nWhichHost == TCP_HOST)
    	{
       		/* Create the memory for the XDR format data representation */
      		xdrmem_create(&l_stReadMessage.m_XDRVariable, l_stReadMessage.m_pXDRData, a_nSize, XDR_DECODE);
      
		/* Convert to the XDR Data representation */
		if(xdr_u_char(&l_stReadMessage.m_XDRVariable, (unsigned char *)a_pDataElement) == FALSE)
		{
			GCLogError("XDR Decoding unsigned character Error\n");
			GC_FREE(l_stReadMessage.m_pXDRData);
			return XDR_ERROR;
		}
	}
	
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(l_stReadMessage.m_pXDRData, a_pDataElement, a_nSize);
      	}

	/* Delete the Memory allocated for storing the XDR data format */
	GC_FREE(l_stReadMessage.m_pXDRData);

	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              Reads a "signed short integer" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which the read has to be done
 *              a_nMessageID - Message Identifier to be read
 *              a_pDataElement - pointer to the data element to be read
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int ReadXDRSignedShortInteger(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
     	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

	MESSAGE l_stReadMessage;

	int l_nErrorCode;

	GC_CALLOC(l_stReadMessage.m_pXDRData, char, a_nSize);
 
	/* Read the XDR Message */
	if((l_nErrorCode = ReadXDRMessage(sock, &l_stReadMessage)) < 0)
	{
		/* Delete the Memory allocated for storing the XDR data format */
		GC_FREE(l_stReadMessage.m_pXDRData);
		return l_nErrorCode;
	}

	if(a_nWhichHost == TCP_HOST)
	{
  		/* Create the memory for the XDR format data representation */
		xdrmem_create(&l_stReadMessage.m_XDRVariable, l_stReadMessage.m_pXDRData, a_nSize, XDR_DECODE);
      
		/* Convert to the XDR Data representation */
		if(xdr_short(&l_stReadMessage.m_XDRVariable, (signed short *)a_pDataElement) == FALSE)
		{
			GCLogError("XDR Decoding signed short integer Error\n");	
			GC_FREE(l_stReadMessage.m_pXDRData);
			return XDR_ERROR;
		}
    	}
	
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(l_stReadMessage.m_pXDRData, a_pDataElement, a_nSize);
	}

	/* Delete the Memory allocated for storing the XDR data format */
	GC_FREE(l_stReadMessage.m_pXDRData);

	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              Reads an "unsigned short integer" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which the read has to be done
 *              a_nMessageID - Message Identifier to be read
 *              a_pDataElement - pointer to the data element to be read
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int ReadXDRUnSignedShortInteger(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
     	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

  	MESSAGE l_stReadMessage;

  	int l_nErrorCode;

  	GC_CALLOC(l_stReadMessage.m_pXDRData, char, a_nSize);

   	/* Read the XDR Message */
  	if((l_nErrorCode = ReadXDRMessage(sock, &l_stReadMessage )) < 0)
    	{
      		/* Delete the Memory allocated for storing the XDR data format */
      		GC_FREE(l_stReadMessage.m_pXDRData);
      		return l_nErrorCode;
    	}

  	if(a_nWhichHost == TCP_HOST)
    	{
        	/* Create the memory for the XDR format data representation */
      		xdrmem_create(&l_stReadMessage.m_XDRVariable, l_stReadMessage.m_pXDRData, a_nSize, XDR_DECODE);
      
      		/* Convert to the XDR Data representation */
      		if(xdr_u_short(&l_stReadMessage.m_XDRVariable, (unsigned short int *)a_pDataElement) == FALSE)
		{
	  		GCLogError("XDR Decoding unsigned short integer Error\n");
	  		GC_FREE(l_stReadMessage.m_pXDRData);
	  		return XDR_ERROR;
		}
    	}
  
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(l_stReadMessage.m_pXDRData, a_pDataElement, a_nSize);
      	}
  
	/* Delete the Memory allocated for storing the XDR data format */
  	GC_FREE(l_stReadMessage.m_pXDRData);

  	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              Reads a "signed integer" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which the read has to be done
 *              a_nMessageID - Message Identifier to be read
 *              a_pDataElement - pointer to the data element to be read
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int ReadXDRSignedInteger(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
     	int sock; 
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

  	MESSAGE l_stReadMessage;

  	int l_nErrorCode;

  	GC_CALLOC(l_stReadMessage.m_pXDRData, char, a_nSize);

  	/* Read the XDR Message */
  	if((l_nErrorCode = ReadXDRMessage(sock, &l_stReadMessage)) < 0)
    	{
      		/* Delete the Memory allocated for storing the XDR data format */
      		GC_FREE(l_stReadMessage.m_pXDRData);
      		return l_nErrorCode;
    	}

  	if(a_nWhichHost == TCP_HOST)
    	{
        	/* Create the memory for the XDR format data representation */
      		xdrmem_create(&l_stReadMessage.m_XDRVariable, l_stReadMessage.m_pXDRData, a_nSize, XDR_DECODE);
      
      		/* Convert to the XDR Data representation */
      		if(xdr_int(&l_stReadMessage.m_XDRVariable, (signed int *)a_pDataElement) == FALSE)
		{
	  		GCLogError("XDR Decoding signed integer Error\n");
	  		GC_FREE(l_stReadMessage.m_pXDRData);
	  		return XDR_ERROR;
		}
	}
  
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(l_stReadMessage.m_pXDRData, a_pDataElement, a_nSize);
      	}

  	/* Delete the Memory allocated for storing the XDR data format */
  	GC_FREE(l_stReadMessage.m_pXDRData);

  	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              Reads an "unsigned integer" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which the read has to be done
 *              a_nMessageID - Message Identifier to be read
 *              a_pDataElement - pointer to the data element to be read
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int ReadXDRUnSignedInteger(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
     	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

  	MESSAGE l_stReadMessage;

  	int l_nErrorCode;

  	GC_CALLOC(l_stReadMessage.m_pXDRData, char, a_nSize);

  	/* Read the XDR Message */
  	if((l_nErrorCode = ReadXDRMessage(sock, &l_stReadMessage)) < 0)
    	{
      		/* Delete the Memory allocated for storing the XDR data format */
      		GC_FREE(l_stReadMessage.m_pXDRData);
      		return l_nErrorCode;
    	}
  
  	if(a_nWhichHost == TCP_HOST)
    	{
        	/* Create the memory for the XDR format data representation */
      		xdrmem_create(&l_stReadMessage.m_XDRVariable, l_stReadMessage.m_pXDRData, a_nSize, XDR_DECODE);
      
      		/* Convert to the XDR Data representation */
      		if(xdr_u_int(&l_stReadMessage.m_XDRVariable, (signed int *)a_pDataElement) == FALSE)
		{
	  		GCLogError("XDR Decoding unsigned integer Error\n");
	  		GC_FREE(l_stReadMessage.m_pXDRData);
	  		return XDR_ERROR;
		}
    	}
  
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(l_stReadMessage.m_pXDRData, a_pDataElement, a_nSize);
      	}
  
	/* Delete the Memory allocated for storing the XDR data format */
  	GC_FREE(l_stReadMessage.m_pXDRData);

  	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              Reads a "signed long integer" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which the read has to be done
 *              a_nMessageID - Message Identifier to be read
 *              a_pDataElement - pointer to the data element to be read
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int ReadXDRSignedLongInteger(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
     	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

  	MESSAGE l_stReadMessage;

  	int l_nErrorCode;

  	GC_CALLOC(l_stReadMessage.m_pXDRData, char, a_nSize);

  	/* Read the XDR Message */
  	if((l_nErrorCode = ReadXDRMessage(sock, &l_stReadMessage)) < 0)
    	{
      		/* Delete the Memory allocated for storing the XDR data format */
      		GC_FREE(l_stReadMessage.m_pXDRData);
      		return l_nErrorCode;
    	}
  
  	if(a_nWhichHost == TCP_HOST)
    	{
        	/* Create the memory for the XDR format data representation */
      		xdrmem_create(&l_stReadMessage.m_XDRVariable, l_stReadMessage.m_pXDRData, a_nSize, XDR_DECODE);
      
      		/* Convert to the XDR Data representation */
      		if(xdr_long(&l_stReadMessage.m_XDRVariable, (signed long int *)a_pDataElement) == FALSE)
		{
	  		GCLogError("XDR Decoding signed long Error\n");
	  		GC_FREE(l_stReadMessage.m_pXDRData);
	  		return XDR_ERROR;
		}
    	}
  
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(l_stReadMessage.m_pXDRData, a_pDataElement, a_nSize);
      	}
  
	/* Delete the Memory allocated for storing the XDR data format */
  	GC_FREE(l_stReadMessage.m_pXDRData);

  	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              Reads an "unsigned long integer" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which the read has to be done
 *              a_nMessageID - Message Identifier to be read
 *              a_pDataElement - pointer to the data element to be read
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int ReadXDRUnSignedLongInteger(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
     	int sock;
     	int a_nMessageID;
    	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

   	MESSAGE l_stReadMessage;

  	int l_nErrorCode;

  	GC_CALLOC(l_stReadMessage.m_pXDRData, char, a_nSize);

  	/* Read the XDR Message */
  	if((l_nErrorCode = ReadXDRMessage(sock, &l_stReadMessage)) < 0)
    	{
      		/* Delete the Memory allocated for storing the XDR data format */
      		GC_FREE(l_stReadMessage.m_pXDRData);
      		return l_nErrorCode;
    	}
  
  	if(a_nWhichHost == TCP_HOST)
    	{
        	/* Create the memory for the XDR format data representation */
      		xdrmem_create(&l_stReadMessage.m_XDRVariable, l_stReadMessage.m_pXDRData, a_nSize, XDR_DECODE);
      
      		/* Convert to the XDR Data representation */
      		if(xdr_u_long(&l_stReadMessage.m_XDRVariable, (unsigned long int *)a_pDataElement) == FALSE)
		{
	  		GCLogError("XDR Decoding unsigned long Error\n");
	  		GC_FREE(l_stReadMessage.m_pXDRData);
	  		return XDR_ERROR;
		}
    	}
  
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(l_stReadMessage.m_pXDRData, a_pDataElement, a_nSize);
      	}
  
	/* Delete the Memory allocated for storing the XDR data format */
  	GC_FREE(l_stReadMessage.m_pXDRData);

  	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              Reads a "string" in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which the read has to be done
 *              a_nMessageID - Message Identifier to be read
 *              a_pDataElement - pointer to the data element to be read
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int ReadXDRString(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
     	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

  	MESSAGE l_stReadMessage;

  	char *l_pszPointerToTheString;

  	int l_nErrorCode;
  	int l_nBytesDecoded;

  	GC_CALLOC(l_stReadMessage.m_pXDRData, char,  LENGTH_FIELD_FOR_XDR_STRING + a_nSize);
  
	/* Read the XDR Message */
  	if((l_nErrorCode = ReadXDRMessage(sock, &l_stReadMessage)) < 0)
    	{
      		/* Delete the Memory allocated for storing the XDR data format */
      		GC_FREE(l_stReadMessage.m_pXDRData);
      		return l_nErrorCode;
    	}

  	if(a_nWhichHost == TCP_HOST)
    	{
        	/* Create the memory for the XDR format data representation */
      		xdrmem_create(&l_stReadMessage.m_XDRVariable, l_stReadMessage.m_pXDRData,  LENGTH_FIELD_FOR_XDR_STRING + a_nSize, XDR_DECODE);
      
      		l_pszPointerToTheString = (char *)a_pDataElement;
      		l_nBytesDecoded =  a_nSize;
      
		/* Convert to the XDR Data representation */
      		if(xdr_string(&l_stReadMessage.m_XDRVariable, (char **)&l_pszPointerToTheString, a_nSize+LENGTH_FIELD_FOR_XDR_STRING) == FALSE)
		{
	  		GCLogError("XDR Decoding String Error\n");
	 	 	GC_FREE(l_stReadMessage.m_pXDRData);
	  		return XDR_ERROR;
		}
      	}
  
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(l_stReadMessage.m_pXDRData, a_pDataElement, a_nSize);
      	}

  	/* Delete the Memory allocated for storing the XDR data format */
  	GC_FREE(l_stReadMessage.m_pXDRData);

  	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              Reads a "float" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which the read has to be done
 *              a_nMessageID - Message Identifier to be read
 *              a_pDataElement - pointer to the data element to be read
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int ReadXDRFloat(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
     	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

  	MESSAGE l_stReadMessage;

  	int l_nErrorCode;

  	GC_CALLOC(l_stReadMessage.m_pXDRData, char, a_nSize);

  	/* Read the XDR Message */
 	if((l_nErrorCode = ReadXDRMessage(sock, &l_stReadMessage)) < 0)
    	{
      		/* Delete the Memory allocated for storing the XDR data format */
      		GC_FREE(l_stReadMessage.m_pXDRData);
      		return l_nErrorCode;
    	}
  
  	if(a_nWhichHost == TCP_HOST)
    	{
        	/* Create the memory for the XDR format data representation */
      		xdrmem_create(&l_stReadMessage.m_XDRVariable, l_stReadMessage.m_pXDRData, a_nSize, XDR_DECODE);
      
      		/* Convert to the XDR Data representation */
      		if(xdr_float(&l_stReadMessage.m_XDRVariable, (float *)a_pDataElement) == FALSE)
		{
	  		GCLogError("XDR Decoding float Error\n");
	  		GC_FREE(l_stReadMessage.m_pXDRData);
	  		return XDR_ERROR;
		}
    	}
  
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(l_stReadMessage.m_pXDRData, a_pDataElement, a_nSize);
      	}
  
	/* Delete the Memory allocated for storing the XDR data format */
 	GC_FREE(l_stReadMessage.m_pXDRData);

  	return l_nErrorCode;
}


/************************************************************************************************************
 * Description : 
 *              Reads a "double" value in XDR format
 *              
 * Arguments   :
 * 		sock - Socket descriptor on which the read has to be done
 *              a_nMessageID - Message Identifier to be read
 *              a_pDataElement - pointer to the data element to be read
 *              a_nSize - size of the data item
 *              a_nWhichHost - Type of the Host TCP/UNIX Host
 *              
 * Return      :
 *              Error Code / 1
 *              
 ************************************************************************************************************/

int ReadXDRDouble(sock, a_nMessageID, a_pDataElement, a_nSize, a_nWhichHost)
  	int sock;
     	int a_nMessageID;
     	void *a_pDataElement;
     	int a_nSize;
     	int a_nWhichHost;
{

  	MESSAGE l_stReadMessage;

  	int l_nErrorCode;

  	GC_CALLOC(l_stReadMessage.m_pXDRData, char, a_nSize);
  
  	/* Read the XDR Message */
  	if((l_nErrorCode = ReadXDRMessage(sock, &l_stReadMessage)) < 0)
    	{
      		/* Delete the Memory allocated for storing the XDR data format */
      		GC_FREE(l_stReadMessage.m_pXDRData);
      		return l_nErrorCode;
    	}

  	if(a_nWhichHost == TCP_HOST)
    	{
        	/* Create the memory for the XDR format data representation */
      		xdrmem_create(&l_stReadMessage.m_XDRVariable, l_stReadMessage.m_pXDRData, a_nSize, XDR_DECODE);
      
      		/* Convert to the XDR Data representation */
      		if(xdr_double(&l_stReadMessage.m_XDRVariable, (double *)a_pDataElement) == FALSE)
		{
	  		GCLogError("XDR Decoding double Error\n");
	  		GC_FREE(l_stReadMessage.m_pXDRData);
	  		return XDR_ERROR;
		}
    	}
  
	else if(a_nWhichHost == UNIXSTREAM_HOST)
      	{
		GC_COPY(l_stReadMessage.m_pXDRData, a_pDataElement, a_nSize);
      	}
  
	/* Delete the Memory allocated for storing the XDR data format */
  	GC_FREE(l_stReadMessage.m_pXDRData);

  	return l_nErrorCode;
}
